#ifndef LOW_PASS_FILTER_H
#define LOW_PASS_FILTER_H

/**
 * @brief Implements a simple low pass filter.
 *
 * @param current_value The current value from the filter.
 * @param new_value The new input to the filter.
 * @return The new output from the filter.
 */
double low_pass_filter(double current_value, double new_value);

#endif
